import unittest
import responses
from unittest.mock import MagicMock
from errors import AnalyticsGraphQLError
from analytics import Analytics, GQL_TRACK_QUERY
import json

API_URL = "http://mock-api/"
API_TOKEN = "<mock-token>"
mock_meta = {"channel": "mock"}


class AnalyitcsTest(unittest.TestCase):
  def test_init_sdk(self):
    analytics = Analytics(api_url=API_URL, api_token=API_TOKEN, meta=mock_meta)
    self.assertIsInstance(analytics, Analytics)

  @responses.activate
  def test_track_request(self):
    analytics = Analytics(api_url=API_URL, api_token=API_TOKEN, meta=mock_meta)
    # Mock response
    responses.add(responses.POST, API_URL, json={'data': {}}, status=200)
    track_result = analytics.track(category='Category',
                                   label='Label',
                                   action={'data': 1},
                                   user_id=1)
    req = responses.calls[0].request
    res = responses.calls[0].response
    self.assertEqual(len(responses.calls), 1)
    self.assertEqual(req.url, API_URL)
    self.assertEqual(req.headers['Content-Type'], 'application/json')
    self.assertEqual(req.headers['Authorization'], 'Bearer ' + API_TOKEN)
    self.assertDictEqual(
        json.loads(req.body), {
            'query': GQL_TRACK_QUERY,
            'variables': {
                "category": 'Category',
                "label": 'Label',
                "action": {
                    'data': 1
                },
                "meta": mock_meta,
                "userId": 1,
                "sessionId": None
            }
        })
    self.assertEqual(res.text, '{"data": {}}')
    self.assertEqual(track_result, True)

  @responses.activate
  def test_track_report_error(self):
    on_error_mock = MagicMock(return_value=None)
    analytics = Analytics(api_url=API_URL,
                          api_token=API_TOKEN,
                          on_error=on_error_mock,
                          meta=mock_meta)
    # Mock response
    responses.add(responses.POST,
                  API_URL,
                  json={'errors': [{
                      'message': '<error>'
                  }]},
                  status=200)
    track_result = analytics.track(category='Category',
                                   label='Label',
                                   action='Action',
                                   user_id=1)
    self.assertEqual(track_result, False)
    on_error_mock.assert_called_once()


if __name__ == '__main__':
  unittest.main()



import requests
import json
from errors import AnalyticsGraphQLError, AnalyticsError

GQL_TRACK_QUERY = """
	mutation js_sdk_track(
		$category: String!
		$label: String!
		$action: jsonb!
		$meta: jsonb!
		$userId: String
		$sessionId: bigint
	) {
		insert_Event(
			objects: {
				category: $category
				label: $label
				action: $action
				meta: $meta
				userId: $userId
				sessionId: $sessionId
			}
		) {
			affected_rows
		}
	}
"""


class Analytics:
  """SDK de Analytics para trackear eventos

    Args:
        api_url (String): A URL de API que o está hospedado servidor GraphQL de Analytics
        api_token (String): O Token de API gerado para acessar a API
        meta (Dict): Os metadados enviados para API
        on_error ((error: AnalyticsError | AnalyticsGraphQLError) -> void): Callback para
          acessar os erros
        log_events (Boolean): Use para categorizar o evento
        session_id (Int): Use para categorizar o evento
  """
  def __init__(self,
               api_url,
               api_token,
               meta,
               on_error=None,
               log_events=False,
               session_id=None):
    self.api_url = api_url
    self.api_token = api_token
    self.meta = meta
    self.on_error = on_error
    self.log_events = log_events
    self.session_id = session_id

  def track(self,
            category,
            label,
            action,
            override_meta=None,
            user_id=None,
            session_id=None):
    """Envie um evento para a API de analytics

    Args:
        category (String): Use para categorizar o evento
        label (String): Use para identificar o evento
        action (any): Use para enriquecer o evento
    """
    userId = user_id

    # Sobrepõe os dados metada enviados à API para esta request
    meta = {} if override_meta else self.meta
    if override_meta:
      meta.update(self.meta).update(override_meta)

    payload = {
        "query": GQL_TRACK_QUERY,
        "variables": {
            "category": category,
            "label": label,
            "action": action,
            "meta": meta,
            "userId": userId,
            "sessionId": session_id
        }
    }
    headers = {
        'content-type': 'application/json',
        "authorization": "Bearer " + self.api_token
    }
    res = requests.post(self.api_url, data=json.dumps(payload), headers=headers)
    res_json = res.json()

    if 'errors' in res_json:
      if self.on_error is not None:
        self.on_error(AnalyticsGraphQLError(res_json['errors'][0]['message']))

      # Vamos retornar False para mostrar que o evento não foi trackeado
      return False

    if not 'data' in res_json:
      if self.on_error is not None:
        self.on_error(AnalyticsError('No data was returned'))

      # Vamos retornar False para mostrar que o evento não foi trackeado
      return False

    return True
